Package: CFilt
Title: Collaborative Filtering Models for Recommendation Systems
Version: 1.0.1
Authors@R: 
  c(person(given = "Jessica",
           family = "Kubrusly",
           role = c("aut", "cre"),
           email = "jessicakubrusly@id.uff.br",
           comment = c(ORCID = "0000-0003-0465-4629")),
    person(given = "Thiago",
           family = "Lima",
           role = "ctb"),
    person(given = "Lucas",
           family = "Oliveira",
           role = "ctb"),
    person(given = "Caio",
           family = "Salviano",
           role = "ctb"))
Description: Implements collaborative filtering methods for recommendation systems based on user-item interaction data. Supports both explicit feedback (ratings) and implicit feedback (consumption). The package uses efficient sparse matrix representations and provides incremental updates for users, items, and similarity structures through an R6 class-based architecture. See Aggarwal (2016) <doi:10.1007/978-3-319-29659-3> for an overview.
License: MIT + file LICENSE
Encoding: UTF-8
Imports: Matrix, R6
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-15 22:38:40 UTC; jessi
Author: Jessica Kubrusly [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-0465-4629>),
  Thiago Lima [ctb],
  Lucas Oliveira [ctb],
  Caio Salviano [ctb]
Maintainer: Jessica Kubrusly <jessicakubrusly@id.uff.br>
Depends: R (>= 3.5.0)
Repository: CRAN
Date/Publication: 2026-05-30 20:10:02 UTC
Built: R 4.6.0; ; 2026-05-30 20:48:56 UTC; unix

Package: CFilt
Title: Collaborative Filtering Models for Recommendation Systems
Version: 1.0.0
Authors@R: 
  c(person(given = "Jessica",
           family = "Kubrusly",
           role = c("aut", "cre"),
           email = "jessicakubrusly@id.uff.br",
           comment = c(ORCID = "0000-0003-0465-4629")),
    person(given = "Thiago",
           family = "Lima",
           role = "ctb"),
    person(given = "Lucas",
           family = "Oliveira",
           role = "ctb"),
    person(given = "Caio",
           family = "Salviano",
           role = "ctb"))
Description: Implements collaborative filtering methods for recommendation systems based on user-item interaction data. Supports both explicit feedback (ratings) and implicit feedback (consumption). The package uses efficient sparse matrix representations and provides incremental updates for users, items, and similarity structures through an R6 class-based architecture. See Aggarwal (2016) <doi:10.1007/978-3-319-29659-3> for an overview.
License: MIT + file LICENSE
Encoding: UTF-8
Imports: Matrix, R6
RoxygenNote: 7.3.3
NeedsCompilation: no
Packaged: 2026-04-25 00:02:20 UTC; jessi
Author: Jessica Kubrusly [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-0465-4629>),
  Thiago Lima [ctb],
  Lucas Oliveira [ctb],
  Caio Salviano [ctb]
Maintainer: Jessica Kubrusly <jessicakubrusly@id.uff.br>
Depends: R (>= 3.5.0)
Repository: CRAN
Date/Publication: 2026-04-25 02:10:02 UTC
Built: R 4.6.0; ; 2026-04-27 08:08:10 UTC; unix

# Generated by roxygen2: do not edit by hand

export(CF)
export(CFbuilder)
export(kclosestitems)
export(topkitems)
export(topkusers)
importFrom(Matrix,Matrix)
importFrom(Matrix,cbind2)
importFrom(Matrix,rbind2)
importFrom(Matrix,sparseMatrix)
importFrom(R6,R6Class)
importFrom(utils,head)
importFrom(utils,setTxtProgressBar)
importFrom(utils,txtProgressBar)

#'@title Recommendation by Collaborative Filtering
#'@description CF is a class of objects that stores
#'information about a recommendation system. This information includes the
#'consumption or rating of each (user, item) pair in the utility matrix MU, the
#'similarities between each pair of users in the similarity matrix SU, the
#'similarities between each pair of items in the similarity matrix SI, the
#'number of items consumed and/or rated by each user in the vector n_aval_u,
#'the number of users who consumed and/or rated each item in the vector
#'n_aval_i, the average rating value of each user in the vector averages_u, the
#'average rating value received by each item in the vector averages_i, the
#'number of items consumed in common by each pair of users in the matrix Int_U,
#'and the number of users in common for each pair of items in the matrix Int_I.
#'The class contains methods such as addNewUser, addNewEmptyUser, deleteUser,
#'addNewItem, addNewEmptyItem, deleteItem, newRating and deleteRating, which
#'modify the object's structure by altering users, items, or consumption data.
#'The class also includes functions such as kClosestItems, topKUsers, and
#'topKItems, which return items to recommend to a user or users to whom an item
#'should be recommended. An object of the CF class is created using the
#'CFBuilder function.
#'@field MU The Utility Matrix, a matrix that contains all the users' ratings.
#'The rows comprise users and the columns, items.
#'@field SU The user similarity matrix.
#'@field SI The item similarity matrix
#'@field IntU A symmetric matrix that records the number of items in common
#'between pairs of users.
#'@field IntI A symmetric matrix that records the number of users in common
#'between pairs of items.umber of items in common that
#'@field averages_u A vector that contains the averages of users' ratings.
#'@field averages_i A vector that contains the averages of items' ratings.
#'@field n_aval_u A vector that stores the number of items rated by each user.
#'@field n_aval_i A vector that stores the number of users who consumed each
#'item.
#'@field datatype A character that indicates the type of data, which can be
#'either "consumption" or "rating".
#'@field similarity  A character string indicating the similarity measure used.
#' It can be "pearson" or "cosine" for rating data, and "jaccard" for consumption data.
#'@field data_0 A data.frame containing the original dataset used to build the object.
#' This corresponds to the input data provided to CFbuilder and is stored for reference.
#'
#'@references
#'\itemize{
#'\item LINDEN, G.; SMITH, B.; YORK, J. Amazon. com recommendations:
#'Item-to-item collaborative filtering. Internet Computing, IEEE, v. 7, n. 1,
#'p. 76-80,2003
#'\item Aggarwal, C. C. (2016). Recommender systems (Vol. 1). Cham: Springer
#'International Publishing.
#'\item Leskovec, J., Rajaraman, A., & Ullman, J. D. (2020). Mining of massive
#'data sets. Cambridge university press.
#'}
#'
#'@seealso \code{\link[CFilt]{CFbuilder}}
#'
#'
#'@author Jessica Kubrusly
#'@importFrom Matrix sparseMatrix rbind2 cbind2 Matrix
#'@importFrom R6 R6Class
#'@export
#'@details
#'This class implements a collaborative filtering system supporting both
#'explicit (ratings) and implicit (consumption) feedback. The internal state
#'is updated incrementally after each operation.
#'@examples
#'data(movies, package = "CFilt")
#'# --- Rating data ---
#'objectCF_r <- CFbuilder(Data = movies[1:500,],Datatype = "rating",
#'similarity = "pearson")
#'dim(objectCF_r$MU)
#'colnames(objectCF_r$MU) #movies Id
#'rownames(objectCF_r$MU) #users Id
#'dim(objectCF_r$SU)
#'dim(objectCF_r$SI)
#'objectCF_r$averages_u
#'objectCF_r$averages_i
#'objectCF_r$n_aval_u
#'objectCF_r$n_aval_i
#'objectCF_r$addnewemptyuser(Id_u = "newuser1")
#'objectCF_r$newrating(Id_u = "newuser1",Id_i = "Frozen",r = 5)
#'objectCF_r$MU["newuser1","Frozen"]
#'objectCF_r$newrating(Id_u = list("newuser1","newuser1","newuser1"),
#'Id_i = list("Thor: The Dark World","The Lego Movie","Despicable Me 2"),
#'r = list(2,3,4))
#'objectCF_r$n_aval_u["newuser1"]
#'objectCF_r$averages_u["newuser1"]
#'objectCF_r$addnewemptyuser(Id_u = "newuser2")
#'objectCF_r$newrating(Id_u = list("newuser2","newuser1"),
#'Id_i = list("Frozen","Her"),r = c(2,1))
#'objectCF_r$addnewemptyuser(Id_u = list("newuser3","newuser4"))
#'objectCF_r$newrating(Id_u = list("newuser3","newuser3","newuser4","newuser4"),
#'Id_i = list("The Lego Movie","Wreck-It Ralph","Fast & Furious 6",
#'"12 Years a Slave"),r = list(4,5,4,2))
#'objectCF_r$addnewemptyitem(Id_i = list("movie1","movie2","movie3"))
#'objectCF_r$newrating(
#'Id_u = list("newuser1","newuser1","newuser1",
#'            "newuser2","newuser2","newuser2",
#'            "newuser3","newuser3","newuser3",
#'            "newuser4","newuser4","newuser4"),
#'Id_i = list("movie1","movie2","movie3",
#'            "movie1","movie2","movie3",
#'            "movie1","movie2","movie3",
#'            "movie1","movie2","movie3"),
#'r = list(4,5,4,2,
#'         1,2,1,1,
#'         4,3,1,2))
#'objectCF_r$MU[,"movie1"]
#'objectCF_r$SI["movie1","movie2"]
#'objectCF_r$SU["newuser2","newuser4"]
#'# --- Consumption data ---
#'objectCF_c <- CFbuilder(Data = movies[1:300,-3],Datatype = "consumption",
#'similarity = "jaccard")
#'objectCF_c$addnewemptyuser(Id_u = list("newuser1","newuser2","newuser3"))
#'objectCF_c$newrating(Id_u = list("newuser1","newuser2","newuser3"),
#'Id_i = list("Frozen","Frozen","Frozen"))
#'objectCF_c$newrating(Id_u = list("newuser1","newuser1","newuser1"),
#'Id_i = list("Gravity","The Wolverine","Iron Man 3"))
#'objectCF_c$addnewemptyitem(Id_i = list("movie1","movie2","movie3"))
#'objectCF_c$newrating(Id_u = list("newuser1","newuser1","newuser2","newuser2",
#'"newuser3"),Id_i = list("movie1","movie2","movie1","movie3","movie3"))
#'objectCF_c$MU[,"movie1"]
#'objectCF_c$SI["movie1","movie2"]
#'objectCF_c$SI["movie1","movie3"]
#'objectCF_c$SI["movie2","movie3"]
#'objectCF_c$SU["newuser1","newuser2"]
#'objectCF_c$SU["newuser2","newuser3"]
#' @export
CF <- R6::R6Class(
  "CF",
  public = list(

    # Fields ----
    MU = NULL,
    SU = NULL,
    SI = NULL,
    IntI = NULL,
    IntU = NULL,
    n_aval_u = NULL,
    n_aval_i = NULL,
    averages_u = NULL,
    averages_i = NULL,
    datatype   = NULL,
    similarity = NULL,
    data_0 = NULL,

    # Methods ----

    #' @description Add a new empty user to the system.
    #' This method creates a new user with no interactions.
    #' @param Id_u A character string (or a list of strings) representing user ID(s).
    #' @return Invisibly returns the updated object.
    addnewemptyuser = function(Id_u) {
      ## Recursion ----
      if (is.list(Id_u)) {
        for (u in Id_u) self$addnewemptyuser(u)
        return(invisible(self))
      }

      ## Validations ----
      if (!is.character(Id_u) || length(Id_u) != 1) {
        stop("*** Id_u must be a single character string. ***")
      }
      if (Id_u %in% rownames(self$MU)) {
        stop(paste0("*** User ", Id_u, " is already registered in the system. ***"))
      }

      ## Ratings ----
      if (self$datatype == "rating") {

        ### Basic Info ----
        N <- ncol(self$MU)
        M <- nrow(self$MU)
        k <- M + 1

        ### Update MU ----
        novo_usuario <- Matrix::sparseMatrix(
          i = integer(0),
          j = integer(0),
          x = numeric(0),
          dims = c(1L, N),
          dimnames = list(Id_u, colnames(self$MU))
        )
        self$MU <- Matrix::rbind2(self$MU, novo_usuario)

        ### Update counters ----
        self$n_aval_u <- c(self$n_aval_u, setNames(0L, Id_u))
        self$averages_u <- c(self$averages_u, setNames(NA_real_, Id_u))

        ### Update IntU ----
        new_row <- Matrix::Matrix(0, nrow = 1, ncol = M, sparse = TRUE)
        self$IntU <- Matrix::rbind2(self$IntU, new_row)
        self$IntU <- Matrix::cbind2(self$IntU, Matrix::Matrix(0, nrow = k, ncol = 1, sparse = TRUE))
        rownames(self$IntU) <- colnames(self$IntU) <- rownames(self$MU)

        ### Update SU ----
        new_col <- Matrix::Matrix(0, nrow = M, ncol = 1, sparse = TRUE)
        self$SU <- Matrix::cbind2(self$SU, new_col)
        self$SU <- Matrix::rbind2(self$SU, Matrix::Matrix(0, nrow = 1, ncol = k, sparse = TRUE))
        self$SU[k, k] <- 1
        rownames(self$SU) <- colnames(self$SU) <- rownames(self$MU)

        return(invisible(self))
      }

      ## Consumption ----
      if (self$datatype == "consumption") {

        ### Basic Info ----
        N <- ncol(self$MU)
        M <- nrow(self$MU)
        k <- M + 1

        ### Update MU ----
        novo_usuario <- Matrix::sparseMatrix(
          i = integer(0),
          j = integer(0),
          x = numeric(0),
          dims = c(1L, N),
          dimnames = list(Id_u, colnames(self$MU))
        )
        self$MU <- Matrix::rbind2(self$MU, novo_usuario)

        ### Update counters ----
        self$n_aval_u <- c(self$n_aval_u, setNames(0L, Id_u))

        ### Update IntU ----
        new_row <- Matrix::Matrix(0, nrow = 1, ncol = M, sparse = TRUE)
        self$IntU <- Matrix::rbind2(self$IntU, new_row)
        self$IntU <- Matrix::cbind2(self$IntU, Matrix::Matrix(0, nrow = k, ncol = 1, sparse = TRUE))
        rownames(self$IntU) <- colnames(self$IntU) <- rownames(self$MU)

        ### Update SU ----
        new_col <- Matrix::Matrix(0, nrow = M, ncol = 1, sparse = TRUE)
        self$SU <- Matrix::cbind2(self$SU, new_col)
        self$SU <- Matrix::rbind2(self$SU, Matrix::Matrix(0, nrow = 1, ncol = k, sparse = TRUE))
        self$SU[k, k] <- 1
        rownames(self$SU) <- colnames(self$SU) <- rownames(self$MU)

        return(invisible(self))
      }

      stop("*** Unknown datatype: must be 'rating' or 'consumption'. ***")
    },

    #' @description Add a new empty item to the system.
    #' @param Id_i A character string (or list of strings) representing item ID(s).
    addnewemptyitem = function(Id_i) {
      ## Recursion ----
      if (is.list(Id_i)) {
        for (i in Id_i) self$addnewemptyitem(i)
        return(invisible(self))
      }

      ## Validations ----
      if (!is.character(Id_i) || length(Id_i) != 1) {
        stop("*** Id_i must be a single character string. ***")
      }
      if (Id_i %in% colnames(self$MU)) {
        stop(paste0("*** Item ", Id_i, " is already registered in the system. ***"))
      }

      ## Rating ----
      if (self$datatype == "rating") {

        ### Basic Info ----
        N <- ncol(self$MU)
        M <- nrow(self$MU)
        k <- N + 1

        ### Update MU ----
        novo_item <- Matrix::sparseMatrix(
          i = integer(0),
          j = integer(0),
          x = numeric(0),
          dims = c(M, 1L),
          dimnames = list(rownames(self$MU), Id_i)
        )
        self$MU <- Matrix::cbind2(self$MU, novo_item)

        ### Update counters ----
        self$n_aval_i <- c(self$n_aval_i, setNames(0L, Id_i))
        self$averages_i <- c(self$averages_i, setNames(NA_real_, Id_i))

        ### Update IntI ----
        new_col <- Matrix::Matrix(0, nrow = N, ncol = 1, sparse = TRUE)
        self$IntI <- Matrix::cbind2(self$IntI, new_col)
        self$IntI <- Matrix::rbind2(self$IntI, Matrix::Matrix(0, nrow = 1, ncol = k, sparse = TRUE))
        rownames(self$IntI) <- colnames(self$IntI) <- colnames(self$MU)

        ### Update SI ----
        new_col <- Matrix::Matrix(0, nrow = N, ncol = 1, sparse = TRUE)
        self$SI <- Matrix::cbind2(self$SI, new_col)
        self$SI <- Matrix::rbind2(self$SI, Matrix::Matrix(0, nrow = 1, ncol = k, sparse = TRUE))
        self$SI[k, k] <- 1
        rownames(self$SI) <- colnames(self$SI) <- colnames(self$MU)

        return(invisible(self))
      }

      ## Consumption ----
      if (self$datatype == "consumption") {

        ### Basic Info ----
        N <- ncol(self$MU)
        M <- nrow(self$MU)
        k <- N + 1

        ### Update MU ----
        novo_item <- Matrix::sparseMatrix(
          i = integer(0),
          j = integer(0),
          x = numeric(0),
          dims = c(M, 1L),
          dimnames = list(rownames(self$MU), Id_i)
        )
        self$MU <- Matrix::cbind2(self$MU, novo_item)

        ### Update counters ----
        self$n_aval_i <- c(self$n_aval_i, setNames(0L, Id_i))

        ### Update IntI ----
        new_col <- Matrix::Matrix(0, nrow = N, ncol = 1, sparse = TRUE)
        self$IntI <- Matrix::cbind2(self$IntI, new_col)
        self$IntI <- Matrix::rbind2(self$IntI, Matrix::Matrix(0, nrow = 1, ncol = k, sparse = TRUE))
        rownames(self$IntI) <- colnames(self$IntI) <- colnames(self$MU)

        ### Update SI ----
        new_col <- Matrix::Matrix(0, nrow = N, ncol = 1, sparse = TRUE)
        self$SI <- Matrix::cbind2(self$SI, new_col)
        self$SI <- Matrix::rbind2(self$SI, Matrix::Matrix(0, nrow = 1, ncol = k, sparse = TRUE))
        self$SI[k, k] <- 1
        rownames(self$SI) <- colnames(self$SI) <- colnames(self$MU)

        return(invisible(self))
      }

      stop("*** Unknown datatype: must be 'rating' or 'consumption'. ***")
    },

    #' @description Add a new rating or consumption.
    #' @param Id_u A character string (or a list of strings) representing user ID(s).
    #' @param Id_i A character string (or a list of strings) representing item ID(s).
    #' @param r A numeric (or a list of numeric) for rating value(s) (only for rating data)
    newrating = function(Id_u, Id_i, r = NULL) {

      if (is.list(Id_u) && is.list(Id_i)) {

        u = length(Id_u)

        if(self$datatype == "rating"){
          for (k in 1:u) {
            self$newrating(Id_u = Id_u[[k]],
                      Id_i = Id_i[[k]],
                      r = r[[k]])
          }
        } else {
          for (k in 1:u) {
            self$newrating(Id_u = Id_u[[k]],
                      Id_i = Id_i[[k]])
          }
        }
      } else {

        if (!is.character(Id_u)) {
          stop("*** Id_u self$MUst be a character. ***")
        }

        if (!is.character(Id_i)) {
          stop("*** Id_i self$MUst be a character. ***")
        }

        M = nrow(self$MU)
        N = ncol(self$MU)

        i <- match(Id_u, rownames(self$MU))
        if (is.na(i)) {
          stop("*** This is not a valid user. ***")
        }

        j <- match(Id_i, colnames(self$MU))
        if (is.na(j)) {
          stop("*** This is not a valid item ***")
        }


        # Dados de Avaliacoes ####
        if (self$datatype == "rating") {

          if (self$MU[i, j]!=0) {
            stop(paste0("*** User ",Id_u," has already consumed item ",Id_i," ***"))
          }

          if (!is.numeric(r)) {
            stop("*** r self$MUst be numeric. ***")
          }


          ## Alteracao self$MU ####
          self$MU[i, j] <<- r


          # Alteracao n_aval_i ####
          self$n_aval_i[j] <<- self$n_aval_i[j] + 1
          # Alteracao n_aval_u ####
          self$n_aval_u[i] <<- self$n_aval_u[i] + 1

          #Alteracao averages_i
          self$averages_i[j] <<- ifelse(
            is.na(self$averages_i[j]),
            r,
            (self$averages_i[j]*(self$n_aval_i[j] - 1) + r)/(self$n_aval_i[j]))


          #Alteracao averages_i
          self$averages_u[i] <<- ifelse(
            is.na(self$averages_u[i]),
            r,
            (self$averages_u[i]*(self$n_aval_u[i] - 1) + r)/(self$n_aval_u[i]))


          # Alteracao self$IntU ####
          self$IntU[i,] <<- self$IntU[i,] + (!is.na(self$MU[,j]))
          self$IntU[,i] <<- self$IntU[,i] + (!is.na(self$MU[,j]))
          self$IntU[i,i] <<- self$IntU[i,i] - 1

          self$IntI[j,] <<- self$IntI[j,] + (!is.na(self$MU[i,]))
          self$IntI[,j] <<- self$IntI[,j] + (!is.na(self$MU[i,]))
          self$IntI[j,j] <<- self$IntI[j,j] - 1


          if(self$similarity=="pearson"){

            s = sapply(X = 1:M,
                       FUN = "pearson",
                       type = "user",
                       i = i,CF = self
            )
          } else {
            if(self$similarity=="cosine"){
              s = sapply(X = 1:M,
                         FUN = "cosine",
                         type = "user",
                         i = i,CF = self
              )
            }
          }

          self$SU[i,] <<- s
          self$SU[,i] <<- s


          if(self$similarity=="pearson"){

            s = sapply(X = 1:N,
                       FUN = "pearson",
                       type = "item",
                       i = j,CF = self
            )
          } else {
            if(self$similarity=="cosine"){
              s = sapply(X = 1:N,
                         FUN = "cosine",
                         type = "item",
                         i = j,CF = self
              )
            }
          }

          self$SI[j,] <<- s
          self$SI[,j] <<- s


        }

        # Dados de Consumo ####
        if (self$datatype == "consumption") {

          if (self$MU[i, j] != 0) {
            stop(paste0("*** User ",Id_u," has already consumed item ",Id_i," ***"))
          }


          ## Alteracao self$MU ####
          self$MU[i, j] <<- 1


          # Alteracao n_aval_i ####
          self$n_aval_i[j] <<- self$n_aval_i[j] + 1


          # Alteracao n_aval_u ####
          self$n_aval_u[i] <<- self$n_aval_u[i] + 1


          # Alteracao self$IntU ####
          self$IntU[i,] <<- self$IntU[i,] + self$MU[,j]
          self$IntU[,i] <<- self$IntU[,i] + self$MU[,j]
          self$IntU[i,i] <<- self$IntU[i,i] - 1

          self$IntI[j,] <<- self$IntI[j,] + self$MU[i,]
          self$IntI[,j] <<- self$IntI[,j] + self$MU[i,]
          self$IntI[j,j] <<- self$IntI[j,j] - 1


          s = sapply(X = 1:M,
                     FUN = "jaccard",
                     type ="user",
                     i    = i,
                     CF = self
          )

          self$SU[i,] <<- s
          self$SU[,i] <<- s


          s = sapply(X = 1:N,
                     FUN = "jaccard",
                     type ="item",
                     i    = j,
                     CF = self
          )

          self$SI[j,] <<- s
          self$SI[,j] <<- s


        }
      }
    },

    #' @description Delete a user from the system.
    #' @param Id_u A character string (or a list of strings) representing user ID(s).
    deleteuser = function(Id_u) {
      ## Recursion ----
      if (is.list(Id_u)) {
        for (k in seq_along(Id_u)) {
          self$deleteuser(Id_u[[k]])
        }
        return(invisible(self))
      }

      ## Validations ----
      if (!is.character(Id_u) || length(Id_u) != 1)
        stop("*** Id_u must be a single character string. ***")

      if (!(Id_u %in% rownames(self$MU)))
        stop("*** No user with this Id ***")

      ## Rating ----
      if (self$datatype == "rating") {

        ### Basic Info ----
        k <- which(rownames(self$MU) == Id_u)
        N <- ncol(self$MU)
        M <- nrow(self$MU)

        user_ratings <- self$MU[k, , drop = FALSE]
        consumed <- as.vector(user_ratings != 0)
        j_idx <- which(consumed)

        ### Update counts ----
        self$n_aval_i[j_idx] <- self$n_aval_i[j_idx] - 1L
        self$n_aval_u <- self$n_aval_u[-k]

        ### Update averages ----
        ratings <- user_ratings@x
        self$averages_u <- self$averages_u[-k]
        if (length(j_idx) > 0) {
          old_avg_i <- self$averages_i[j_idx]
          new_count_i <- self$n_aval_i[j_idx]
          self$averages_i[j_idx] <- ifelse(new_count_i > 0,
                                           (old_avg_i * (new_count_i + 1) - ratings) / new_count_i,
                                           NA
          )
        }

        ### Update IntI ----
        if (length(j_idx) > 0) {
          self$IntI[j_idx, j_idx] <- self$IntI[j_idx, j_idx] - 1L
        }

        ### Update SI ----
        if (length(j_idx) > 0) {
          for (idx in j_idx) {
            if (self$similarity == "pearson") {
              s <- pearson(type = "item",
                           i = idx,
                           j = 1:ncol(self$MU),
                           CF = self)
            } else {
              s <- cosine(type = "item",
                          i = idx,
                          j = 1:ncol(self$MU),
                          CF = self)
            }
            self$SI[idx, ] <- s
            self$SI[, idx] <- s
          }
        }

        ### Update MU ----
        self$MU <- self$MU[-k, , drop = FALSE]

        ### Update IntU ----
        self$IntU <- self$IntU[-k, -k, drop = FALSE]
        rownames(self$IntU) <- colnames(self$IntU) <- rownames(self$MU)

        ### Update SU ----
        self$SU <- self$SU[-k, -k, drop = FALSE]
        rownames(self$SU) <- colnames(self$SU) <- rownames(self$MU)

        return(invisible(self))
      }

      ## Consumption ----
      if (self$datatype == "consumption") {

        ### Basic Info ----
        k <- which(rownames(self$MU) == Id_u)
        N <- ncol(self$MU)
        M <- nrow(self$MU)

        user_consumo <- self$MU[k, , drop = FALSE]
        consumed <- as.vector(user_consumo != 0)
        j_idx <- which(consumed)

        ### Update counts ----
        self$n_aval_i[j_idx] <- self$n_aval_i[j_idx] - 1L
        self$n_aval_u <- self$n_aval_u[-k]

        ### Update IntI ----
        if (length(j_idx) > 0) {
          self$IntI[j_idx, j_idx] <- self$IntI[j_idx, j_idx] - 1L
        }

        ### Update SI ----
        if (length(j_idx) > 0) {
          for (idx in j_idx) {
            s <- jaccard(type = "item",
                         i = idx,
                         j = 1:ncol(self$MU),
                         CF = self)
            self$SI[idx, ] <- s
            self$SI[, idx] <- s
          }
        }

        ### Update MU ----
        self$MU <- self$MU[-k, , drop = FALSE]

        ### Update IntU and SU ----
        self$IntU <- self$IntU[-k, -k, drop = FALSE]
        self$SU <- self$SU[-k, -k, drop = FALSE]
        rownames(self$IntU) <- colnames(self$IntU) <- rownames(self$MU)
        rownames(self$SU) <- colnames(self$SU) <- rownames(self$MU)

        return(invisible(self))
      }

      stop("*** Unknown datatype: must be 'rating' or 'consumption'. ***")
    },

    #' @description Delete an item from the system.
    #' @param Id_i A character string (or a list of strings) representing item ID(s).
    deleteitem = function(Id_i) {
      ## Recursion ----
      if (is.list(Id_i)) {
        for (k in seq_along(Id_i)) {
          self$deleteitem(Id_i[[k]])
        }
        return(invisible(self))
      }

      ## Validations ----
      if (!is.character(Id_i) || length(Id_i) != 1)
        stop("*** Id_i must be a single character string. ***")

      if (!(Id_i %in% colnames(self$MU)))
        stop(paste0("*** Item ", Id_i, " is not registered in the system. ***"))

      ## Rating ----
      if (self$datatype == "rating") {
        ### Basic Info ----
        k <- which(colnames(self$MU) == Id_i)
        N <- ncol(self$MU)
        M <- nrow(self$MU)

        item_ratings <- self$MU[, k, drop = FALSE]
        consumed <- as.vector(item_ratings != 0)
        i_idx <- which(consumed)

        ### Update counts ----
        self$n_aval_u[i_idx] <- self$n_aval_u[i_idx] - 1L
        self$n_aval_i <- self$n_aval_i[-k]

        ### Update averages ----
        ratings <- item_ratings@x
        self$averages_i <- self$averages_i[-k]
        if (length(i_idx) > 0) {
          old_avg_u <- self$averages_u[i_idx]
          new_count_u <- self$n_aval_u[i_idx]
          self$averages_u[i_idx] <- ifelse(new_count_u > 0,
                                           (old_avg_u * (new_count_u + 1) - ratings) / new_count_u,
                                           NA
          )
        }

        ### Update IntI ----
        self$IntI <- self$IntI[-k, -k, drop = FALSE]

        ### Update SI ----
        self$SI <- self$SI[-k, -k, drop = FALSE]

        ### Update MU ----
        self$MU <- self$MU[, -k, drop = FALSE]

        ### Update IntU ----
        if (length(i_idx) > 0) {
          self$IntU[i_idx, i_idx] <- self$IntU[i_idx, i_idx] - 1L
        }

        ### Update SU ----
        if (length(i_idx) > 0) {
          users <- rownames(self$MU)[i_idx]

          if (self$similarity == "pearson") {
            for (user_i in users) {
              s <- sapply(colnames(self$SU),
                          FUN = "pearson",
                          type = "user",
                          i = user_i,
                          CF = self)
              self$SU[user_i, ] <- s
              self$SU[, user_i] <- s
            }
          } else {
            for (user_i in users) {
              s <- sapply(colnames(self$SU),
                          FUN = "cosine",
                          type = "user",
                          i = user_i,
                          CF = self)
              self$SU[user_i, ] <- s
              self$SU[, user_i] <- s
            }
          }
        }


        return(invisible(self))
      }

      ## Consumption ----
      if (self$datatype == "consumption") {
        ### Basic Info ----
        k <- which(colnames(self$MU) == Id_i)
        N <- ncol(self$MU)
        M <- nrow(self$MU)

        item_consumo <- self$MU[, k, drop = FALSE]
        consumed <- as.vector(item_consumo != 0)
        i_idx <- which(consumed)

        ### Update counts ----
        self$n_aval_u[i_idx] <- self$n_aval_u[i_idx] - 1L
        self$n_aval_i <- self$n_aval_i[-k]

        ### Update MU ----
        self$MU <- self$MU[, -k, drop = FALSE]

        ### Update IntI and SI ----
        self$IntI <- self$IntI[-k, -k, drop = FALSE]
        self$SI <- self$SI[-k, -k, drop = FALSE]
        rownames(self$IntI) <- colnames(self$IntI) <- colnames(self$MU)
        rownames(self$SI) <- colnames(self$SI) <- colnames(self$MU)

        ### Update IntU ----
        if (length(i_idx) > 0) {
          self$IntU[i_idx, i_idx] <- self$IntU[i_idx, i_idx] - 1L
        }

        ### Update SU ----
        if (length(i_idx) > 0) {
          inter <- as.matrix(self$IntU)
          union <- outer(self$n_aval_u, self$n_aval_u, "+") - inter
          self$SU[i_idx, ] <- ifelse(union[i_idx, ] > 0, inter[i_idx, ] / union[i_idx, ], 0)
          self$SU[, i_idx] <- Matrix::t(self$SU[i_idx, , drop = FALSE])
        }

        return(invisible(self))
      }

      stop("*** Unknown datatype: must be 'rating' or 'consumption'. ***")
    },

    #' @description Delete a rating or consumption.
    #' @param Id_u A character string (or a list of strings) representing user ID(s).
    #' @param Id_i A character string (or a list of strings) representing item ID(s).
    deleterating = function(Id_u, Id_i) {

      ## Recursion ----
      if (is.list(Id_u) && is.list(Id_i)) {
        mapply(function(u, i) self$deleterating(u, i),
               Id_u, Id_i, SIMPLIFY = FALSE, USE.NAMES = FALSE)
        return(invisible(self))
      }

      ## Validations ----
      if (!is.character(Id_u) || length(Id_u) != 1) stop("*** Id_u must be a single character. ***")
      if (!is.character(Id_i) || length(Id_i) != 1) stop("*** Id_i must be a single character. ***")

      if (!(Id_u %in% rownames(self$MU))) stop("*** This is not a valid user. ***")
      if (!(Id_i %in% colnames(self$MU))) stop("*** This is not a valid item. ***")

      ## Rating ----
      if (self$datatype == "rating") {

        i <- match(Id_u, rownames(self$MU))
        j <- match(Id_i, colnames(self$MU))

        if (self$MU[i, j] == 0) {
          stop("*** This rating does not exist. ***")
        }

        ### Update counts ----
        self$n_aval_i[j] <- self$n_aval_i[j] - 1L
        self$n_aval_u[i] <- self$n_aval_u[i] - 1L

        ### Update averages ----
        self$averages_u[i] <- (self$averages_u[i]*(self$n_aval_u[i] + 1L) - self$MU[i, j])/(self$n_aval_u[i])
        self$averages_i[j] <- (self$averages_i[j]*(self$n_aval_i[j] + 1L) - self$MU[i, j])/(self$n_aval_i[j])

        ### Update IntU ----
        # Update user intersection matrix by removing contributions of this rating
        col_j_bin <- (self$MU[, j] != 0)
        self$IntU[i, ] <- self$IntU[i, ] - col_j_bin
        self$IntU[, i] <- self$IntU[, i] - col_j_bin
        self$IntU[i, i] <- self$IntU[i, i] + 1L

        ### Update IntI ----
        # Update item intersection matrix similarly
        row_i_bin <- (self$MU[i, ] != 0)
        self$IntI[j, ] <- self$IntI[j, ] - row_i_bin
        self$IntI[, j] <- self$IntI[, j] - row_i_bin
        self$IntI[j, j] <- self$IntI[j, j] + 1L

        ### Update MU ----
        # Remove rating from user-item matrix
        self$MU[i, j] <- 0

        ### Update SU ----
        if (self$similarity == "pearson") {
          s <- sapply(X = list(1:nrow(self$MU)),
                      FUN = "pearson",
                      type = "user",
                      i = i,
                      CF = self
          )
        } else if (self$similarity == "cosine") {
          s <- sapply(X = list(1:nrow(self$MU)),
                      FUN = "cosine",
                      type = "user",
                      i = i,
                      CF = self
          )
        }

        self$SU[i, ] <- s
        self$SU[, i] <- s

        ### Update SI ----
        if (self$similarity == "pearson") {
          s <- sapply(X = list(1:ncol(self$MU)),
                      FUN = "pearson",
                      type = "item",
                      i = j,
                      CF = self
          )
        } else if (self$similarity == "cosine") {
          s <- sapply(X = list(1:ncol(self$MU)),
                      FUN = "cosine",
                      type = "item",
                      i = j,
                      CF = self
          )
        }

        self$SI[j, ] <- s
        self$SI[, j] <- s

        return(invisible(NULL))

      }

      ## Consumption ----
      if (self$datatype == "consumption") {

        i <- match(Id_u, rownames(self$MU))
        j <- match(Id_i, colnames(self$MU))

        if (self$MU[i, j] == 0) {
          stop("*** This rating does not exist. ***")
        }

        ### Update counts ----
        # Decrement total ratings counts for user and item
        self$n_aval_i[j] <- self$n_aval_i[j] - 1
        self$n_aval_u[i] <- self$n_aval_u[i] - 1

        ### Update IntU ----
        # Update user intersection matrix by removing contributions of this rating
        self$IntU[i, ] <- self$IntU[i, ] - self$MU[, j]
        self$IntU[, i] <- self$IntU[, i] - self$MU[, j]
        self$IntU[i, i] <- self$n_aval_u[i]

        ### Update IntI ----
        # Update item intersection matrix similarly
        self$IntI[j, ] <- self$IntI[j, ] - self$MU[i, ]
        self$IntI[, j] <- self$IntI[, j] - self$MU[i, ]
        self$IntI[j, j] <- self$n_aval_i[j]

        ### Update MU ----
        # Remove rating from user-item matrix
        self$MU[i, j] <- 0

        ### Update SU ----
        # Recalculate user similarity for affected user
        intersection <- self$IntU[i, ]
        union <- self$n_aval_u[i] + self$n_aval_u - intersection
        s_user <- intersection / union
        s_user[is.nan(s_user)] <- 0
        s_user[i] <- 1
        self$SU[i, ] <- s_user
        self$SU[, i] <- s_user

        ### Update SI ----
        # Recalculate item similarity for affected item
        intersection_i <- self$IntI[j, ]
        union_i <- self$n_aval_i[j] + self$n_aval_i - intersection_i
        s_item <- intersection_i / union_i
        s_item[is.nan(s_item)] <- 0
        s_item[j] <- 1
        self$SI[j, ] <- s_item
        self$SI[, j] <- s_item

        return(invisible(NULL))
      }
    }
  )
)

#' @title K Closest Items
#'
#' @description
#' Returns the k most similar items to a given item based on the item-item
#' similarity matrix of a collaborative filtering model.
#'
#' @param CF An object of class \code{CF} created by \code{\link{CFbuilder}}.
#' @param Id_i A character string representing the item ID.
#' @param k A positive integer indicating the number of similar items to return.
#' Default is 10.
#'
#' @return A character vector containing the IDs of the k most similar items.
#'
#' @details
#' The similarity between items is obtained from the item similarity matrix
#' stored in \code{CF$SI}. The item itself is excluded from the result.
#'
#' @examples
#' data(movies, package = "CFilt")
#'
#' CF1 <- CFbuilder(movies[1:200, ], Datatype = "rating")
#'
#' # Find the 5 items most similar to a given item
#' kclosestitems(CF1, Id_i = "Frozen", k = 5)
#'
#' @seealso \code{\link{CFbuilder}}, \code{\link{topkitems}}, \code{\link{topkusers}}
#'
#' @export
kclosestitems <- function (CF, Id_i, k = 10) {

  # Validations ----
  if (!is.character(Id_i) || length(Id_i) != 1) {
    stop("*** 'Id_i' must be a single character string. ***")
  }
  if (!is.numeric(k) || length(k) != 1 || k <= 0) {
    stop("*** 'k' must be a single positive number. ***")
  }
  if (!(Id_i %in% colnames(CF$MU))) {
    stop("*** This is not a valid item. ***")
  }

  # Calculation ----
  MU <- CF$MU
  SI <- CF$SI

  j <- match(Id_i, colnames(MU))

  s <- SI[, j]
  s[j] <- 0
  ind <- head(order(s, decreasing = T, na.last = NA), k)

  return(colnames(SI)[ind])
}

#' Movie ratings by users
#'
#' A dataset containing 7276 ratings for 50 movies by 526 users.
#'
#' @format A data frame with 7276 rows and 3 variables:
#' \describe{
#' \item{IdUsers}{Users identifier. Numbers 1 to 526.}
#' \item{IdItems}{Movies identifier. Movies names.}
#'   \item{Ratings}{Movie ratings by users. Ratings from 1 to 5 (Likert scale).}
#' }
#'
#' @source Giglio (2014)
#' @name movies
#' @docType data
#' @keywords datasets
NULL

cosine <- function(CF, type = c("user", "item"), i, j) {
  
  # Validation ----
  if (CF$datatype != "rating") {
    stop("*** only defined for ratings data. ***")
  }
  
  if (!type %in% c("user", "item")) {
    stop("*** type must be 'user' or 'item' ***")
  }
  
  # Single comparison ----
  if (length(j) == 1) {
    
    if (type == "user") {
      v <- CF$MU[i, ]
      w <- CF$MU[j, ]
    } else {
      v <- CF$MU[, i]
      w <- CF$MU[, j]
    }
    
    s <- ifelse(
      sum(v * v, na.rm = TRUE) == 0 | sum(w * w, na.rm = TRUE) == 0,
      0,
      sum(v * w, na.rm = TRUE) / (sqrt(sum(v * v, na.rm = TRUE) * sum(w * w, na.rm = TRUE)))
    )
    
    return(s)
  }
  
  # Multiple comparisons ----
  else {
    
    if (type == "user") {
      v <- CF$MU[i, ]
      w <- CF$MU[j, ]
      
      s <- apply(w, MARGIN = 1, function(x) {
        ifelse(
          sum(v * v, na.rm = TRUE) == 0 | sum(x * x, na.rm = TRUE) == 0,
          0,
          sum(v * x, na.rm = TRUE) / (sqrt(sum(v * v, na.rm = TRUE) * sum(x * x, na.rm = TRUE)))
        )
      })
      
      return(s)
      
    } else {
      v <- CF$MU[, i]
      w <- CF$MU[, j]
      
      s <- apply(w, MARGIN = 2, function(x) {
        ifelse(
          sum(v * v, na.rm = TRUE) == 0 | sum(x * x, na.rm = TRUE) == 0,
          0,
          sum(v * x, na.rm = TRUE) / (sqrt(sum(v * v, na.rm = TRUE) * sum(x * x, na.rm = TRUE)))
        )
      })
    }
    
    return(s)
  }
}

#' @title Top-K Item Recommendation
#'
#' @description
#' Returns the top-k items to recommend for a given user based on a
#' collaborative filtering model.
#'
#' @param CF An object of class \code{CF}.
#' @param Id_u A character string representing the user ID.
#' @param k A positive integer indicating the number of items to recommend.
#' Default is 10.
#' @param type A character string indicating the recommendation strategy:
#' \itemize{
#'   \item \code{"user"}: user-based collaborative filtering
#'   \item \code{"item"}: item-based collaborative filtering
#' }
#'
#' @return A character vector containing the IDs of the top-k recommended items.
#'
#' @details
#' For \code{type = "user"}, recommendations are computed based on similarities
#' between users. For \code{type = "item"}, recommendations are computed based
#' on similarities between items.
#'
#' Only items not yet consumed/rated by the user are considered.
#'
#' @examples
#' data(movies, package = "CFilt")
#'
#' CF1 <- CFbuilder(movies[1:200, ], Datatype = "rating")
#'
#' # Recommend users for an item using user-based CF
#' topkitems(CF1, Id_u = "1", k = 5, type = "user")
#'
#' # Recommend users for an item using item-based CF
#' topkitems(CF1, Id_u = "1", k = 3, type = "item")
#' 
#' CF2 <- CFbuilder(movies[1:200,-3])
#' 
#' # Recommend users for an item using user-based CF
#' topkitems(CF2, Id_u = "1", k = 5, type = "user")
#'
#' # Recommend users for an item using item-based CF
#' topkitems(CF2, Id_u = "1", k = 3, type = "item")
#'
#' @seealso \code{\link{CFbuilder}}, \code{\link{topkitems}}
#'
#' @export
topkitems <- function(CF, Id_u, k = 10, type = "user") {
  
  # Validations ----
  if (!is.character(Id_u) || length(Id_u) != 1) {
    stop("*** 'Id_u' must be a single character string. ***")
  }
  
  if (!is.numeric(k) || length(k) != 1 || k <= 0) {
    stop("*** 'k' must be a single positive number. ***")
  }
  
  if (!type %in% c("user", "item")) {
    stop("*** 'type' must be either 'user' or 'item'. ***")
  }
  
  MU <- CF$MU
  
  if (!Id_u %in% rownames(MU)) {
    stop("*** This is not a valid user. ***")
  }
  
  i <- match(Id_u, rownames(MU))
  
  # Calculation ----
  if (type == "user") {
    
    s <- CF$SU[, i]
    notas <- Matrix::crossprod(s, MU) / sum(s)
    notas <- as.numeric(notas)
    
  } else {  # type == "item"
    s <- CF$SI
    v <- MU[i, , drop = FALSE]
    
    notas <- v %*% s
    denom <- Matrix::colSums(s, na.rm = TRUE)
    
    denom[denom == 0] <- NA
    notas <- as.numeric(notas) / denom
  }
  
  nao_avaliados <- MU[i, ] == 0
  
  notas <- notas[nao_avaliados]
  itens <- colnames(MU)[nao_avaliados]
  
  ind <- head(order(notas, decreasing = TRUE, na.last = TRUE), k)
  return(itens[ind])
}

CFupdate <- function(obj_CF, Data_new){
  
  # Validations ----
  if (!inherits(obj_CF, "CF")) {
    stop("*** 'cf_object' must be a CF object created by CFbuilder() ***")
  }
  
  if (!is.data.frame(Data_new)) {
    stop("*** 'Data_new' must be a dataframe object ***")
  }
  
  if (obj_CF$datatype == "rating" && ncol(Data_new) != 3) {
    stop("*** CF object has datatype='ratings' but Data_new must have 3 columns ***")
  }
  
  if (obj_CF$datatype == "consumption" && ncol(Data_new) != 2) {
    stop("*** CF object has datatype='consumption' but Data_new must have 2 columns ***")
  }
  
  # Setup ----
  data_new <- as.data.frame(Data_new)
  data_old <- as.data.frame(obj_CF$data_0)
  
  key_old <- interaction(data_old[[1]], data_old[[2]], drop = TRUE)
  key_new <- interaction(data_new[[1]], data_new[[2]], drop = TRUE)
  
  pos <- match(key_new, key_old)
  
  # Add Rown ----
  idx_add <- which(is.na(pos))
  if (length(idx_add) > 0) {
    data_old <- rbind(data_old, data_new[idx_add, ])
    key_old  <- c(key_old, key_new[idx_add])
  }
  
  # Update row ----
  if (obj_CF$datatype == "rating") {
    idx_upd <- which(!is.na(pos))
    if (length(idx_upd) > 0) {
      data_old[pos[idx_upd], 3] <- data_new[idx_upd, 3]
    }
  }
  
  # Re-build ----
  Data <- data_old
  
  return(
    CFbuilder(
      Data = Data,
      Datatype = obj_CF$datatype,
      similarity = obj_CF$similarity
    )
  )
}

pearson <- function(CF, type = c("user", "item"), i, j) {
  
  
  # Validation ----
  if (CF$datatype != "rating") {
    stop("*** only defined for rating data. ***")
  }
  
  if (!type %in% c("user", "item")) {
    stop("*** type must be 'user' or 'item' ***")
  }
  
  # Single comparison ----
  
  if(length(j)==1){
    if(type=="user"){
      v = CF$MU[i, , drop = FALSE]
      v = as.vector(v) - CF$averages_u[i]
      v[CF$MU[i, ] == 0] = NA
      
      w = CF$MU[j, , drop = FALSE]
      w = as.vector(w) - CF$averages_u[j]
      w[CF$MU[j, ] == 0] = NA
      
      s = ifelse(sum(v*v,na.rm=T)==0 | sum(w*w,na.rm=T)==0,0,
                 sum(v*w,na.rm=T)/(sqrt(sum(v*v,na.rm=T)*sum(w*w,na.rm=T))))
      return(s)
    } else {
      if(type=="item"){
        v = CF$MU[, i, drop = FALSE]
        v = as.vector(v) - CF$averages_i[i]
        v[CF$MU[,i] == 0] = NA
        
        w = CF$MU[,j, drop = FALSE]
        w = as.vector(w) - CF$averages_i[j]
        w[CF$MU[,j] == 0] = NA
        
        s = ifelse(sum(v*v,na.rm=T)==0 | sum(w*w,na.rm=T)==0,0,
                   sum(v*w,na.rm=T)/(sqrt(sum(v*v,na.rm=T)*sum(w*w,na.rm=T))))
        return(s)
        
      }
    }
    stop("*** type must be item or user ***")
  }
  
  # Multiple comparisons ----
  
  else { 
    if(type=="user"){
      v = CF$MU[i, , drop = FALSE]
      v = as.vector(v) - CF$averages_u[i]
      v[CF$MU[i, ] == 0] = NA
      
      W = CF$MU[j, , drop = FALSE]
      
      W_centered = W - matrix(CF$averages_u[j],
                              nrow = length(j),
                              ncol = ncol(CF$MU),
                              byrow = FALSE)
      
      W_centered[W == 0] = NA
      
      s = apply(W_centered, MARGIN = 1, function(x){
        ifelse(sum(v*v,na.rm=T)==0 | sum(x*x,na.rm=T)==0,0,
               sum(v*x,na.rm=T)/(sqrt(sum(v*v,na.rm=T)*sum(x*x,na.rm=T))))
      })
      return(s)
    } else {
      if(type=="item"){
        v = CF$MU[, i, drop = FALSE]
        v = as.vector(v) - CF$averages_i[i]
        v[CF$MU[, i] == 0] = NA
        
        W = CF$MU[, j, drop = FALSE]
        
        W_centered = W - matrix(CF$averages_i[j],
                                nrow = nrow(CF$MU),
                                ncol = length(j),
                                byrow = TRUE)
        
        W_centered[W == 0] = NA
        
        s = apply(W_centered, MARGIN = 2, function(x){
          ifelse(sum(v*v,na.rm=T)==0 | sum(x*x,na.rm=T)==0,0,
                 sum(v*x,na.rm=T)/(sqrt(sum(v*v,na.rm=T)*sum(x*x,na.rm=T))))
        })
        return(s)
      }
    }
    stop("*** type must be item or user ***")
  }
}

jaccard <- function(CF,type,i,j){
  
  if(CF$datatype != "consumption"){
    stop("Only for consumption data")
  }
  
  # Computes Jaccard similarity between two items
  if(type=="item"){
    
    n1 <- CF$n_aval_i[i]
    n2 <- CF$n_aval_i[j]
    n12 <- CF$IntI[i,j]
    union <- n1 + n2 - n12
    return(n12/union)
    
  } else {
    
    # Computes Jaccard similarity between two users
    if(type=="user"){
      
      n1 <- CF$n_aval_u[i]
      n2 <- CF$n_aval_u[j]
      n12 <- CF$IntU[i,j]
      union <- n1 + n2 - n12
      return(n12/union)
      
    }
  }
  stop("*** type must be item or user ***")
}

#'@title Build a Collaborative Filtering Object
#'@description
#'Creates an object of class \code{CF} from a dataset of user-item interactions.
#'The dataset can represent either explicit ratings or implicit consumption.
#'@param Data A data.frame containing:
#'\itemize{
#' \item 2 columns: (user, item) for consumption data
#' \item 3 columns: (user, item, rating) for rating data
#'}
#'@param Datatype A character string indicating the type of data:
#'\code{"consumption"} or \code{"rating"}.
#'Default is inferred from number of columns in \code{Data}.
#'@param similarity A character string indicating the similarity measure:
#'\itemize{
#' \item \code{"jaccard"} for consumption data
#' \item \code{"pearson"} or \code{"cosine"} for rating data
#'}
#'Default is chosen based on \code{Datatype}.
#'@return An object of class \code{CF}.
#'@examples
#'data(movies, package = "CFilt")
#'# --- Rating data ---
#'CF1 <- CFbuilder(Data = movies[1:300,],Datatype = "rating",
#'similarity = "pearson")
#'CF1_ <- CFbuilder(Data = movies[1:300,])
#'CF2 <- CFbuilder(Data = movies[1:300,],Datatype = "rating",
#'similarity = "cosine")
#'CF2_ <- CFbuilder(Data = movies[1:300,],similarity = "cosine")
#'# --- Consumption data ---
#'CF3 <- CFbuilder(Data = movies[1:300,-3],Datatype = "consumption",
#'similarity = "jaccard")
#'CF3_ <- CFbuilder(Data = movies[1:300,-3])
#'@importFrom utils head txtProgressBar setTxtProgressBar
#'@export

CFbuilder <- function(Data,
                      Datatype = ifelse(ncol(Data) == 2, "consumption", "rating"),
                      similarity = ifelse(Datatype == "consumption", "jaccard", "pearson")) {

  # Validations ----
  if (!is.data.frame(Data)) {
    stop("*** Data must be a dataframe object ***")
  }

  if (!(ncol(Data) %in% c(2, 3))) {
    stop("*** Datatype must have 2 (consumption) or 3 (rating) columns ***")
  }

  if (!Datatype %in% c("consumption", "rating")) {
    stop("*** Datatype must be 'consumption' or 'rating' ***")
  }


  # Setup ----

  Data <- as.data.frame(Data)
  name_u <- unique(as.character(Data[[1]]))
  name_i <- unique(as.character(Data[[2]]))
  m <- length(name_u)
  n <- length(name_i)

  obj_CF <- CF$new()
  obj_CF$datatype <- Datatype
  obj_CF$similarity <- similarity

  row_idx <- as.integer(factor(Data[[1]], levels = name_u))
  col_idx <- as.integer(factor(Data[[2]], levels = name_i))

  obj_CF$data_0 <- Data

  # Consumption ----

  if (obj_CF$datatype == "consumption" && ncol(Data) == 2) {

    ## jaccard ----

    if (obj_CF$similarity == "jaccard") {

      ### MU ----
      # Build user-item consumption matrix

      message("Step 1 of 3: Building MU")
      pb <- utils::txtProgressBar(min = 0, max = 3, style = 3)
      utils::setTxtProgressBar(pb, 0)

      row_idx <- as.integer(factor(Data[[1]], levels = name_u))
      col_idx <- as.integer(factor(Data[[2]], levels = name_i))
      utils::setTxtProgressBar(pb, 1)

      obj_CF$MU <- Matrix::sparseMatrix(
        i = row_idx,
        j = col_idx,
        x = 1,
        dims = c(m, n),
        dimnames = list(name_u, name_i)
      )
      utils::setTxtProgressBar(pb, 2)

      obj_CF$n_aval_u <- Matrix::rowSums(obj_CF$MU)
      obj_CF$n_aval_i <- Matrix::colSums(obj_CF$MU)

      utils::setTxtProgressBar(pb, 3)
      close(pb)

      ### SU ----
      # Build user-user Jaccard similarity

      message("Step 2 of 3: Building SU")
      pb <- utils::txtProgressBar(min = 0, max = 4, style = 3)
      utils::setTxtProgressBar(pb, 0)

      intersection_u <- Matrix::tcrossprod(obj_CF$MU)
      utils::setTxtProgressBar(pb, 1)

      nu <- obj_CF$n_aval_u
      inter_map_u <- Matrix::summary(intersection_u)
      i_idx <- inter_map_u$i
      j_idx <- inter_map_u$j
      x_vals <- inter_map_u$x
      union_u <- nu[i_idx] + nu[j_idx] - x_vals

      utils::setTxtProgressBar(pb, 2)
      jaccard_vals_u <- x_vals / union_u
      utils::setTxtProgressBar(pb, 3)

      obj_CF$SU <- Matrix::sparseMatrix(
        i = i_idx,
        j = j_idx,
        x = jaccard_vals_u,
        dims = dim(intersection_u),
        symmetric = TRUE,
        dimnames = dimnames(intersection_u)
      )
      obj_CF$IntU <- intersection_u

      utils::setTxtProgressBar(pb, 4)
      close(pb)

      ### SI ----
      # Build item-item Jaccard similarity

      message("Step 3 of 3: Building SI")
      pb <- utils::txtProgressBar(min = 0, max = 4, style = 3)
      utils::setTxtProgressBar(pb, 0)

      intersection_i <- Matrix::crossprod(obj_CF$MU)
      utils::setTxtProgressBar(pb, 1)

      ni <- obj_CF$n_aval_i
      inter_map_i <- Matrix::summary(intersection_i)
      i_idx <- inter_map_i$i
      j_idx <- inter_map_i$j
      x_vals <- inter_map_i$x
      union_i <- ni[i_idx] + ni[j_idx] - x_vals

      utils::setTxtProgressBar(pb, 2)
      jaccard_vals_i <- x_vals / union_i
      utils::setTxtProgressBar(pb, 3)

      obj_CF$SI <- Matrix::sparseMatrix(
        i = i_idx,
        j = j_idx,
        x = jaccard_vals_i,
        dims = dim(intersection_i),
        symmetric = TRUE,
        dimnames = dimnames(intersection_i)
      )
      obj_CF$IntI <- intersection_i

      utils::setTxtProgressBar(pb, 4)
      close(pb)
    }
  }


  # Ratings ----
  else if (obj_CF$datatype == "rating" && ncol(Data) == 3) {

    ## MU ----
    message("Step 1 of 3: Building MU")
    pb <- utils::txtProgressBar(min = 0, max = 1, style = 3)
    utils::setTxtProgressBar(pb, 0)

    ratings <- as.numeric(Data[[3]])
    ratings[ratings == 0 & !is.na(ratings)] <- .Machine$double.xmin
    ratings[is.na(ratings)] <- 0L

    obj_CF$MU <- Matrix::sparseMatrix(
      i = row_idx,
      j = col_idx,
      x = ratings,
      dims = c(m, n),
      dimnames = list(name_u, name_i)
    )
    utils::setTxtProgressBar(pb, 1)
    close(pb)

    ## pearson ----

    if (obj_CF$similarity == "pearson") {

      binary_MU <- sign(obj_CF$MU)
      obj_CF$n_aval_u <- Matrix::rowSums(binary_MU)
      obj_CF$n_aval_i <- Matrix::colSums(binary_MU)
      obj_CF$averages_u <- Matrix::rowSums(obj_CF$MU)/obj_CF$n_aval_u
      obj_CF$averages_i <- Matrix::colSums(obj_CF$MU)/obj_CF$n_aval_i

      ### SU ----
      message("Step 2 of 3: Building SU")
      pb <- utils::txtProgressBar(min = 0, max = 3, style = 3)
      utils::setTxtProgressBar(pb, 0)

      trip <- Matrix::summary(obj_CF$MU)
      M_u <- Matrix::sparseMatrix(
        i = trip$i,
        j = trip$j,
        x = obj_CF$averages_u[trip$i],
        dims = dim(obj_CF$MU),
        dimnames = dimnames(obj_CF$MU)
      )
      utils::setTxtProgressBar(pb, 1)

      MU_centered_u <- obj_CF$MU - M_u

      numer_u <- Matrix::tcrossprod(MU_centered_u)
      utils::setTxtProgressBar(pb, 2)

      norms_u <- sqrt(Matrix::rowSums(MU_centered_u^2))
      norm_prod <- norms_u %o% norms_u

      s_u <- Matrix::summary(numer_u)
      numv <- s_u$x
      denv <- norm_prod[cbind(s_u$i, s_u$j)]
      pe_u <- ifelse(denv > 0, numv / denv, 0)

      obj_CF$SU <- Matrix::sparseMatrix(
        i = s_u$i, j = s_u$j, x = pe_u,
        dims = dim(numer_u), symmetric = TRUE,
        dimnames = dimnames(numer_u)
      )
      obj_CF$IntU <- Matrix::tcrossprod(binary_MU)

      utils::setTxtProgressBar(pb, 3)
      close(pb)

      ### SI ----
      message("Step 3 of 3: Building SI")
      pb <- utils::txtProgressBar(min = 0, max = 3, style = 3)
      utils::setTxtProgressBar(pb, 0)

      M_i <- Matrix::sparseMatrix(
        i = trip$i,
        j = trip$j,
        x = obj_CF$averages_i[trip$j],
        dims = dim(obj_CF$MU),
        dimnames = dimnames(obj_CF$MU)
      )
      utils::setTxtProgressBar(pb, 1)

      MU_centered_i <- obj_CF$MU - M_i
      numer_i <- Matrix::crossprod(MU_centered_i)
      utils::setTxtProgressBar(pb, 2)

      norms_i <- sqrt(Matrix::colSums(MU_centered_i^2))
      norm_prod <- norms_i %o% norms_i

      s_i <- Matrix::summary(numer_i)
      numv <- s_i$x
      denv <- norm_prod[cbind(s_i$i, s_i$j)]
      pe_i <- ifelse(denv > 0, numv / denv, 0)

      obj_CF$SI <- Matrix::sparseMatrix(
        i = s_i$i, j = s_i$j, x = pe_i,
        dims = dim(numer_i), symmetric = TRUE,
        dimnames = dimnames(numer_i)
      )
      obj_CF$IntI <- Matrix::crossprod(binary_MU)

      utils::setTxtProgressBar(pb, 3)
      close(pb)
    }

    ## cosine ----
    else if (obj_CF$similarity == "cosine") {

      binary_MU <- sign(obj_CF$MU)

      ### SU ----
      message("Step 2 of 3: Building SU")
      pb <- utils::txtProgressBar(min = 0, max = 2, style = 3)
      utils::setTxtProgressBar(pb, 0)

      numer_u <- Matrix::tcrossprod(obj_CF$MU)
      norms_u <- sqrt(Matrix::rowSums(obj_CF$MU * obj_CF$MU))
      norm_prod <- norms_u %o% norms_u
      utils::setTxtProgressBar(pb, 1)

      s_u <- Matrix::summary(numer_u)
      numv <- s_u$x
      denv <- norm_prod[cbind(s_u$i, s_u$j)]
      cosu <- ifelse(denv > 0, numv / denv, 0)

      obj_CF$SU <- Matrix::sparseMatrix(
        i = s_u$i, j = s_u$j, x = cosu,
        dims = dim(numer_u), symmetric = TRUE,
        dimnames = dimnames(numer_u)
      )
      obj_CF$IntU <- Matrix::tcrossprod(binary_MU)

      utils::setTxtProgressBar(pb, 2)
      close(pb)

      ### SI ----
      message("Step 3 of 3: Building SI")
      pb <- utils::txtProgressBar(min = 0, max = 2, style = 3)
      utils::setTxtProgressBar(pb, 0)

      numer_i <- Matrix::crossprod(obj_CF$MU)
      norms_i <- sqrt(Matrix::colSums(obj_CF$MU * obj_CF$MU))
      norm_prod <- norms_i %o% norms_i
      utils::setTxtProgressBar(pb, 1)

      s_i <- Matrix::summary(numer_i)
      numv <- s_i$x
      denv <- norm_prod[cbind(s_i$i, s_i$j)]
      cosi <- ifelse(denv > 0, numv / denv, 0)

      obj_CF$SI <- Matrix::sparseMatrix(
        i = s_i$i, j = s_i$j, x = cosi,
        dims = dim(numer_i), symmetric = TRUE,
        dimnames = dimnames(numer_i)
      )
      obj_CF$IntI <- Matrix::crossprod(binary_MU)

      utils::setTxtProgressBar(pb, 2)
      close(pb)
    }
  }

  # Return ----
  return(obj_CF)
}

#' @title Top-K User Recommendation for an Item
#'
#' @description
#' Returns the top-k users to whom a given item should be recommended,
#' based on a collaborative filtering model.
#'
#' @param CF An object of class \code{CF} created by \code{\link{CFbuilder}}.
#' @param Id_i A character string representing the item ID.
#' @param k A positive integer indicating the number of users to return.
#' Default is 10.
#' @param type A character string indicating the recommendation strategy:
#' \itemize{
#'   \item \code{"user"}: user-based collaborative filtering
#'   \item \code{"item"}: item-based collaborative filtering
#' }
#'
#' @return A character vector containing the IDs of the top-k users
#' for whom the item is recommended.
#'
#' @details
#' For \code{type = "user"}, recommendations are based on similarities
#' between users. For \code{type = "item"}, recommendations are based on
#' similarities between items.
#'
#' Only users who have not yet consumed/rated the item are considered.
#'
#' @examples
#' data(movies, package = "CFilt")
#'
#' CF1 <- CFbuilder(movies[1:200, ], Datatype = "rating")
#'
#' # Recommend users for an item using user-based CF
#' topkusers(CF1, Id_i = "Frozen", k = 5, type = "user")
#'
#' # Recommend users for an item using item-based CF
#' topkusers(CF1, Id_i = "Frozen", k = 5, type = "item")
#' 
#' CF2 <- CFbuilder(movies[1:200,-3])
#' 
#' # Recommend users for an item using user-based CF
#' topkusers(CF2, Id_i = "Frozen", k = 5, type = "user")
#'
#' # Recommend users for an item using item-based CF
#' topkusers(CF2, Id_i = "Frozen", k = 3, type = "item")
#'
#' @seealso \code{\link{CFbuilder}}, \code{\link{topkitems}}
#'
#' @export

topkusers <- function(CF, Id_i, k = 10, type = "user") {
  
  # Validations ----
  if (!is.character(Id_i) || length(Id_i) != 1) {
    stop("*** 'Id_i' must be a single character string. ***")
  }
  
  if (!is.numeric(k) || length(k) != 1 || k <= 0) {
    stop("*** 'k' must be a single positive number. ***")
  }
  
  if (!type %in% c("user", "item")) {
    stop("*** 'type' must be either 'user' or 'item'. ***")
  }
  
  MU <- CF$MU
  if (!Id_i %in% colnames(MU)) {
    stop("*** This is not a valid item. ***")
  }
  
  j <- match(Id_i, colnames(MU))
  
  # Calculation ----
  if (type == "user") {
    s <- CF$SU
    v <- MU[, j]
    notas <- as.numeric(Matrix::crossprod(s, v)) / Matrix::colSums(s)
    
  } else {
    s <- CF$SI[, j]
    notas <- as.numeric(MU %*% s) / sum(s)
  }
  
  avaliados <- MU[, j] != 0
  notas[avaliados] <- NA
  

  ind <- head(order(notas, decreasing = TRUE, na.last = NA), k)
  return(rownames(MU)[ind])
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/topkitems.R
\name{topkitems}
\alias{topkitems}
\title{Top-K Item Recommendation}
\usage{
topkitems(CF, Id_u, k = 10, type = "user")
}
\arguments{
\item{CF}{An object of class \code{CF}.}

\item{Id_u}{A character string representing the user ID.}

\item{k}{A positive integer indicating the number of items to recommend.
Default is 10.}

\item{type}{A character string indicating the recommendation strategy:
\itemize{
\item \code{"user"}: user-based collaborative filtering
\item \code{"item"}: item-based collaborative filtering
}}
}
\value{
A character vector containing the IDs of the top-k recommended items.
}
\description{
Returns the top-k items to recommend for a given user based on a
collaborative filtering model.
}
\details{
For \code{type = "user"}, recommendations are computed based on similarities
between users. For \code{type = "item"}, recommendations are computed based
on similarities between items.

Only items not yet consumed/rated by the user are considered.
}
\examples{
data(movies, package = "CFilt")

CF1 <- CFbuilder(movies[1:200, ], Datatype = "rating")

# Recommend users for an item using user-based CF
topkitems(CF1, Id_u = "1", k = 5, type = "user")

# Recommend users for an item using item-based CF
topkitems(CF1, Id_u = "1", k = 3, type = "item")

CF2 <- CFbuilder(movies[1:200,-3])

# Recommend users for an item using user-based CF
topkitems(CF2, Id_u = "1", k = 5, type = "user")

# Recommend users for an item using item-based CF
topkitems(CF2, Id_u = "1", k = 3, type = "item")

}
\seealso{
\code{\link{CFbuilder}}, \code{\link{topkitems}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CFbuilder.R
\name{CFbuilder}
\alias{CFbuilder}
\title{Build a Collaborative Filtering Object}
\usage{
CFbuilder(
  Data,
  Datatype = ifelse(ncol(Data) == 2, "consumption", "rating"),
  similarity = ifelse(Datatype == "consumption", "jaccard", "pearson")
)
}
\arguments{
\item{Data}{A data.frame containing:
\itemize{
\item 2 columns: (user, item) for consumption data
\item 3 columns: (user, item, rating) for rating data
}}

\item{Datatype}{A character string indicating the type of data:
\code{"consumption"} or \code{"rating"}.
Default is inferred from number of columns in \code{Data}.}

\item{similarity}{A character string indicating the similarity measure:
\itemize{
\item \code{"jaccard"} for consumption data
\item \code{"pearson"} or \code{"cosine"} for rating data
}
Default is chosen based on \code{Datatype}.}
}
\value{
An object of class \code{CF}.
}
\description{
Creates an object of class \code{CF} from a dataset of user-item interactions.
The dataset can represent either explicit ratings or implicit consumption.
}
\examples{
data(movies, package = "CFilt")
# --- Rating data ---
CF1 <- CFbuilder(Data = movies[1:300,],Datatype = "rating",
similarity = "pearson")
CF1_ <- CFbuilder(Data = movies[1:300,])
CF2 <- CFbuilder(Data = movies[1:300,],Datatype = "rating",
similarity = "cosine")
CF2_ <- CFbuilder(Data = movies[1:300,],similarity = "cosine")
# --- Consumption data ---
CF3 <- CFbuilder(Data = movies[1:300,-3],Datatype = "consumption",
similarity = "jaccard")
CF3_ <- CFbuilder(Data = movies[1:300,-3])
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/kclosestitems.R
\name{kclosestitems}
\alias{kclosestitems}
\title{K Closest Items}
\usage{
kclosestitems(CF, Id_i, k = 10)
}
\arguments{
\item{CF}{An object of class \code{CF} created by \code{\link{CFbuilder}}.}

\item{Id_i}{A character string representing the item ID.}

\item{k}{A positive integer indicating the number of similar items to return.
Default is 10.}
}
\value{
A character vector containing the IDs of the k most similar items.
}
\description{
Returns the k most similar items to a given item based on the item-item
similarity matrix of a collaborative filtering model.
}
\details{
The similarity between items is obtained from the item similarity matrix
stored in \code{CF$SI}. The item itself is excluded from the result.
}
\examples{
data(movies, package = "CFilt")

CF1 <- CFbuilder(movies[1:200, ], Datatype = "rating")

# Find the 5 items most similar to a given item
kclosestitems(CF1, Id_i = "Frozen", k = 5)

}
\seealso{
\code{\link{CFbuilder}}, \code{\link{topkitems}}, \code{\link{topkusers}}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/movies.R
\docType{data}
\name{movies}
\alias{movies}
\title{Movie ratings by users}
\format{
A data frame with 7276 rows and 3 variables:
\describe{
\item{IdUsers}{Users identifier. Numbers 1 to 526.}
\item{IdItems}{Movies identifier. Movies names.}
\item{Ratings}{Movie ratings by users. Ratings from 1 to 5 (Likert scale).}
}
}
\source{
Giglio (2014)
}
\description{
A dataset containing 7276 ratings for 50 movies by 526 users.
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CF.R
\name{CF}
\alias{CF}
\title{Recommendation by Collaborative Filtering}
\description{
CF is a class of objects that stores
information about a recommendation system. This information includes the
consumption or rating of each (user, item) pair in the utility matrix MU, the
similarities between each pair of users in the similarity matrix SU, the
similarities between each pair of items in the similarity matrix SI, the
number of items consumed and/or rated by each user in the vector n_aval_u,
the number of users who consumed and/or rated each item in the vector
n_aval_i, the average rating value of each user in the vector averages_u, the
average rating value received by each item in the vector averages_i, the
number of items consumed in common by each pair of users in the matrix Int_U,
and the number of users in common for each pair of items in the matrix Int_I.
The class contains methods such as addNewUser, addNewEmptyUser, deleteUser,
addNewItem, addNewEmptyItem, deleteItem, newRating and deleteRating, which
modify the object's structure by altering users, items, or consumption data.
The class also includes functions such as kClosestItems, topKUsers, and
topKItems, which return items to recommend to a user or users to whom an item
should be recommended. An object of the CF class is created using the
CFBuilder function.
}
\details{
This class implements a collaborative filtering system supporting both
explicit (ratings) and implicit (consumption) feedback. The internal state
is updated incrementally after each operation.
}
\examples{
data(movies, package = "CFilt")
# --- Rating data ---
objectCF_r <- CFbuilder(Data = movies[1:500,],Datatype = "rating",
similarity = "pearson")
dim(objectCF_r$MU)
colnames(objectCF_r$MU) #movies Id
rownames(objectCF_r$MU) #users Id
dim(objectCF_r$SU)
dim(objectCF_r$SI)
objectCF_r$averages_u
objectCF_r$averages_i
objectCF_r$n_aval_u
objectCF_r$n_aval_i
objectCF_r$addnewemptyuser(Id_u = "newuser1")
objectCF_r$newrating(Id_u = "newuser1",Id_i = "Frozen",r = 5)
objectCF_r$MU["newuser1","Frozen"]
objectCF_r$newrating(Id_u = list("newuser1","newuser1","newuser1"),
Id_i = list("Thor: The Dark World","The Lego Movie","Despicable Me 2"),
r = list(2,3,4))
objectCF_r$n_aval_u["newuser1"]
objectCF_r$averages_u["newuser1"]
objectCF_r$addnewemptyuser(Id_u = "newuser2")
objectCF_r$newrating(Id_u = list("newuser2","newuser1"),
Id_i = list("Frozen","Her"),r = c(2,1))
objectCF_r$addnewemptyuser(Id_u = list("newuser3","newuser4"))
objectCF_r$newrating(Id_u = list("newuser3","newuser3","newuser4","newuser4"),
Id_i = list("The Lego Movie","Wreck-It Ralph","Fast & Furious 6",
"12 Years a Slave"),r = list(4,5,4,2))
objectCF_r$addnewemptyitem(Id_i = list("movie1","movie2","movie3"))
objectCF_r$newrating(
Id_u = list("newuser1","newuser1","newuser1",
           "newuser2","newuser2","newuser2",
           "newuser3","newuser3","newuser3",
           "newuser4","newuser4","newuser4"),
Id_i = list("movie1","movie2","movie3",
           "movie1","movie2","movie3",
           "movie1","movie2","movie3",
           "movie1","movie2","movie3"),
r = list(4,5,4,2,
        1,2,1,1,
        4,3,1,2))
objectCF_r$MU[,"movie1"]
objectCF_r$SI["movie1","movie2"]
objectCF_r$SU["newuser2","newuser4"]
# --- Consumption data ---
objectCF_c <- CFbuilder(Data = movies[1:300,-3],Datatype = "consumption",
similarity = "jaccard")
objectCF_c$addnewemptyuser(Id_u = list("newuser1","newuser2","newuser3"))
objectCF_c$newrating(Id_u = list("newuser1","newuser2","newuser3"),
Id_i = list("Frozen","Frozen","Frozen"))
objectCF_c$newrating(Id_u = list("newuser1","newuser1","newuser1"),
Id_i = list("Gravity","The Wolverine","Iron Man 3"))
objectCF_c$addnewemptyitem(Id_i = list("movie1","movie2","movie3"))
objectCF_c$newrating(Id_u = list("newuser1","newuser1","newuser2","newuser2",
"newuser3"),Id_i = list("movie1","movie2","movie1","movie3","movie3"))
objectCF_c$MU[,"movie1"]
objectCF_c$SI["movie1","movie2"]
objectCF_c$SI["movie1","movie3"]
objectCF_c$SI["movie2","movie3"]
objectCF_c$SU["newuser1","newuser2"]
objectCF_c$SU["newuser2","newuser3"]
}
\references{
\itemize{
\item LINDEN, G.; SMITH, B.; YORK, J. Amazon. com recommendations:
Item-to-item collaborative filtering. Internet Computing, IEEE, v. 7, n. 1,
p. 76-80,2003
\item Aggarwal, C. C. (2016). Recommender systems (Vol. 1). Cham: Springer
International Publishing.
\item Leskovec, J., Rajaraman, A., & Ullman, J. D. (2020). Mining of massive
data sets. Cambridge university press.
}
}
\seealso{
\code{\link[CFilt]{CFbuilder}}
}
\author{
Jessica Kubrusly
}
\section{Public fields}{
\if{html}{\out{<div class="r6-fields">}}
\describe{
\item{\code{MU}}{The Utility Matrix, a matrix that contains all the users' ratings.
The rows comprise users and the columns, items.}

\item{\code{SU}}{The user similarity matrix.}

\item{\code{SI}}{The item similarity matrix}

\item{\code{IntU}}{A symmetric matrix that records the number of items in common
between pairs of users.}

\item{\code{IntI}}{A symmetric matrix that records the number of users in common
between pairs of items.umber of items in common that}

\item{\code{averages_u}}{A vector that contains the averages of users' ratings.}

\item{\code{averages_i}}{A vector that contains the averages of items' ratings.}

\item{\code{n_aval_u}}{A vector that stores the number of items rated by each user.}

\item{\code{n_aval_i}}{A vector that stores the number of users who consumed each
item.}

\item{\code{datatype}}{A character that indicates the type of data, which can be
either "consumption" or "rating".}

\item{\code{similarity}}{A character string indicating the similarity measure used.
It can be "pearson" or "cosine" for rating data, and "jaccard" for consumption data.}

\item{\code{data_0}}{A data.frame containing the original dataset used to build the object.
This corresponds to the input data provided to CFbuilder and is stored for reference.}
}
\if{html}{\out{</div>}}
}
\section{Methods}{
\subsection{Public methods}{
\itemize{
\item \href{#method-CF-addnewemptyuser}{\code{CF$addnewemptyuser()}}
\item \href{#method-CF-addnewemptyitem}{\code{CF$addnewemptyitem()}}
\item \href{#method-CF-newrating}{\code{CF$newrating()}}
\item \href{#method-CF-deleteuser}{\code{CF$deleteuser()}}
\item \href{#method-CF-deleteitem}{\code{CF$deleteitem()}}
\item \href{#method-CF-deleterating}{\code{CF$deleterating()}}
\item \href{#method-CF-clone}{\code{CF$clone()}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-CF-addnewemptyuser"></a>}}
\if{latex}{\out{\hypertarget{method-CF-addnewemptyuser}{}}}
\subsection{Method \code{addnewemptyuser()}}{
Add a new empty user to the system.
This method creates a new user with no interactions.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{CF$addnewemptyuser(Id_u)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{Id_u}}{A character string (or a list of strings) representing user ID(s).}
}
\if{html}{\out{</div>}}
}
\subsection{Returns}{
Invisibly returns the updated object.
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-CF-addnewemptyitem"></a>}}
\if{latex}{\out{\hypertarget{method-CF-addnewemptyitem}{}}}
\subsection{Method \code{addnewemptyitem()}}{
Add a new empty item to the system.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{CF$addnewemptyitem(Id_i)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{Id_i}}{A character string (or list of strings) representing item ID(s).}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-CF-newrating"></a>}}
\if{latex}{\out{\hypertarget{method-CF-newrating}{}}}
\subsection{Method \code{newrating()}}{
Add a new rating or consumption.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{CF$newrating(Id_u, Id_i, r = NULL)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{Id_u}}{A character string (or a list of strings) representing user ID(s).}

\item{\code{Id_i}}{A character string (or a list of strings) representing item ID(s).}

\item{\code{r}}{A numeric (or a list of numeric) for rating value(s) (only for rating data)}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-CF-deleteuser"></a>}}
\if{latex}{\out{\hypertarget{method-CF-deleteuser}{}}}
\subsection{Method \code{deleteuser()}}{
Delete a user from the system.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{CF$deleteuser(Id_u)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{Id_u}}{A character string (or a list of strings) representing user ID(s).}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-CF-deleteitem"></a>}}
\if{latex}{\out{\hypertarget{method-CF-deleteitem}{}}}
\subsection{Method \code{deleteitem()}}{
Delete an item from the system.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{CF$deleteitem(Id_i)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{Id_i}}{A character string (or a list of strings) representing item ID(s).}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-CF-deleterating"></a>}}
\if{latex}{\out{\hypertarget{method-CF-deleterating}{}}}
\subsection{Method \code{deleterating()}}{
Delete a rating or consumption.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{CF$deleterating(Id_u, Id_i)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{Id_u}}{A character string (or a list of strings) representing user ID(s).}

\item{\code{Id_i}}{A character string (or a list of strings) representing item ID(s).}
}
\if{html}{\out{</div>}}
}
}
\if{html}{\out{<hr>}}
\if{html}{\out{<a id="method-CF-clone"></a>}}
\if{latex}{\out{\hypertarget{method-CF-clone}{}}}
\subsection{Method \code{clone()}}{
The objects of this class are cloneable with this method.
\subsection{Usage}{
\if{html}{\out{<div class="r">}}\preformatted{CF$clone(deep = FALSE)}\if{html}{\out{</div>}}
}

\subsection{Arguments}{
\if{html}{\out{<div class="arguments">}}
\describe{
\item{\code{deep}}{Whether to make a deep clone.}
}
\if{html}{\out{</div>}}
}
}
}
